
import random, color, moves, inanimate
from pimobj import Object, Animate, Wall

class ObjDaemon (Object, Animate):

	def __init__( s, world, appearances, probab ):
		s.world = world
		s.appearances = appearances
		s.probab = probab
		s.world.add_anim( s )

	def proact( s, roundnr ):
		if random.randint( 0, roundnr ) > s.probab: return
		x = random.randint( 0, s.world.dim_x - 1 )
		y = random.randint( 0, s.world.dim_y - 1 )
		if s.world.outbound( x, y ): return
		app = random.choice( s.appearances )
		if app == 'O': inanimate.Boulder( s.world, x, y )
		elif app == '=': inanimate.Block( s.world, x, y )
		elif app == '#': s.world.add_obj( Wall(), x, y )
		elif app == '*': Train( s.world, x, y, moves.up )
		elif app == '&': inanimate.Bomb( s.world, x, y )
		elif app == '^': inanimate.Trigger( s.world, x, y )
		else: inanimate.Decor( s.world, x, y, app, 0 )

class Train (Object, Animate):

	def __init__( s, world, x, y, direction ):
		s.world = world
		s.d = direction
		s.x = x
		s.y = y
		world.add_obj( s, x, y )
		world.add_anim( s )

	def proact( s, roundnr ):
		if roundnr % 3: return
		try: s.x, s.y = moves.move_in_world( s, s.world, s.x, s.y, s.d )
		except moves.NonPermeable: s.d = moves.cw( s.d )

	def draw( s, scr, x, y ):
		scr.addch( y, x, '*', color.clut[14] )

	def permeable( s, o ):
		return False

class Bullet (Object, Animate):

	def __init__( s, world, x, y, direction ):
		s.world = world
		s.d = direction
		s.x = x
		s.y = y
		world.add_obj( s, x, y )
		world.add_anim( s )

	def proact( s, roundnr ):
		try: s.x, s.y = moves.move_in_world( s, s.world, s.x, s.y, s.d )
		except moves.NonPermeable:
			s.world.rm_obj( s, s.x, s.y )
			s.world.rm_anim( s )

	def draw( s, scr, x, y ):
		scr.addch( y, x, '.' )

class WallDaemon (Object, Animate):

	def __init__( s, world, x, y, direction, steps, rank ):
		s.world = world
		s.d = direction
		s.x = x
		s.y = y
		s.wall = Wall()
		s.rank = rank
		s.dowall = not random.randint( 0, 3 )
		s.steps = s.origsteps = steps
		s.spawntime = random.randint( -50, -3 )
		world.add_obj( s, x, y )
		world.add_anim( s )

	def proact( s, roundnr ):
		s.steps = s.steps - 1
		if s.steps == s.spawntime and s.rank > 0:
			WallDaemon( s.world, s.x, s.y, moves.cw( s.d ),
				s.origsteps, s.rank - 1 )
			WallDaemon( s.world, s.x, s.y, moves.ccw( s.d ),
				s.origsteps, s.rank - 1 )
		if s.steps < s.spawntime: s.annihilate()
		if s.steps <= 0: return
		ox, oy = s.x, s.y
		try: s.x, s.y = moves.move_in_world( s, s.world, s.x, s.y, s.d )
		except moves.NonPermeable: s.annihilate()
		if s.dowall: s.world.add_obj( s.wall, ox, oy )

	def permeable( s, obj ):
		return not isinstance( obj, WallDaemon )

	def collide( s, o, d ):
		s.annihilate()

	def annihilate( s ):
		s.world.rm_obj( s, s.x, s.y )
		s.world.rm_anim( s )

	def draw( s, scr, x, y ):
		scr.addch( y, x, '#', color.clut[3] )

class SampleDaemon (Object, Animate):

	def __init__( s, world, x, y, samples, prob ):
		s.world = world
		s.x = x
		s.y = y
		s.prob = prob
		s.steps = 15
		s.d = moves.up
		s.samples = samples
		world.add_obj( s, x, y )
		world.add_anim( s )

	def proact( s, roundnr ):
		if roundnr % 6: return
		s.steps = s.steps - 1
		if s.steps < -9:
			s.steps = random.randint( 0, 10 )
			s.d = random.randint( 1, 4 )
		if s.steps == 0 and s.samples \
		   and not random.randint( 0, s.prob ):
			inanimate.Sample( s.world, s.x, s.y,
					random.choice( s.samples ),
					random.randint( 0, 15 ))
		if s.steps <= 0: return
		try: s.x, s.y = moves.move_in_world( s, s.world, s.x, s.y, s.d )
		except moves.NonPermeable: s.steps = 0

	def collide( s, o, d ):
		try: s.x, s.y = moves.move_in_world( s, s.world, s.x, s.y, d )
		except moves.NonPermeable: pass

	def draw( s, scr, x, y ):
		scr.addch( y, x, '@', color.clut[6] )

