
import color, moves, player, animate, sound
from pimobj import Object

class Decor (Object):

	def __init__( s, world, x, y, appearance, pitch ):
		s.pitch = pitch
		s.world = world
		s.appearance = appearance
		world.add_obj( s, x, y )

	def pickable( s, o ):
		return True

	def draw( s, scr, x, y ):
		scr.addch( y, x, s.appearance, color.clut[s.pitch] )

	def play( s, pitch = None ):
		if pitch != None: s.pitch = pitch

class Sample (Object):

	appearances = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'

	def __init__( s, world, x, y, sample, pitch ):
		s.app = s.appearances[hash( sample ) % len( s.appearances )]
		s.pitch = pitch
		s.world = world
		s.sample = sample
		world.add_obj( s, x, y )

	def pickable( s, o ):
		return True

	def draw( s, scr, x, y ):
		scr.addch( y, x, s.app, color.clut[s.pitch] )

	def play( s, pitch = None ):
		if pitch != None: s.pitch = pitch
		sound.play( s.sample, s.pitch )

class Block (Object):

	def __init__( s, world, x, y ):
		s.world = world
		s.x = x
		s.y = y
		world.add_obj( s, x, y )

	def pickable( s, o ):
		return True

	def permeable( s, o ):
		return isinstance( o, player.Player )

	def draw( s, scr, x, y ):
		scr.addch( y, x, '=', color.clut[11] )

class Boulder (Object):

	def __init__( s, world, x, y ):
		s.world = world
		s.x = x
		s.y = y
		world.add_obj( s, x, y )

	def permeable( s, obj ):
		return False

	def collide( s, obj, d ):
		try: s.x, s.y = moves.move_in_world( s, s.world, s.x, s.y, d )
		except moves.NonPermeable: pass

	def draw( s, scr, x, y ):
		scr.addch( y, x, 'O', color.clut[4] )
	
class Bomb (Object):

	def __init__( s, world, x, y ):
		s.world = world
		s.x = x
		s.y = y
		world.add_obj( s, x, y )

	def permeable( s, obj ):
		return False

	def collide( s, obj, d ):
		animate.Bullet( s.world, s.x, s.y, moves.up )
		animate.Bullet( s.world, s.x, s.y, moves.dn )
		animate.Bullet( s.world, s.x, s.y, moves.lt )
		animate.Bullet( s.world, s.x, s.y, moves.rt )
		s.world.rm_obj( s, s.x, s.y )

	def draw( s, scr, x, y ):
		scr.addch( y, x, '&', color.clut[13] )

class Trigger (Object):

	def __init__( s, world, x, y ):
		s.world = world
		s.x = x
		s.y = y
		world.add_obj( s, x, y )

	def collide( s, obj, d ):
		if s.x == None or s.y == None: return
		animate.Bullet( s.world, s.x, s.y, moves.ccw( d ))

	def move_notify( s, x, y ):
		s.x = x
		s.y = y

	def pickable( s, obj ):
		return True

	def draw( s, scr, x, y ):
		scr.addch( y, x, '^', color.clut[9] )

